<?php

namespace Tests\Feature;

use App\Models\User;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Tests\TestCase;

class ProfileTest extends TestCase
{
    use RefreshDatabase;
    
    public function test_show() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/auth/profile');
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "user" => [
                "id",
                "username",
                "email",
                "created_at",
                "updated_at"
            ]
        ]);
    }
    
    public function test_delete() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->delete('/api/auth/profile');
        $response->assertStatus(204);
        $this->assertDatabaseCount('users', 0);
    }
    
    public function test_update_wrong_email() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/auth/profile', [
            "email" => "jaaj"
        ]);
        $response->assertStatus(422);
    }
    
    public function test_update_same_email() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/auth/profile', [
            "email" => "test@localhost"
        ]);
        $response->assertStatus(204);
    }
    
    public function test_update_duplicate_email() {
        $user = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/auth/profile', [
            "email" => "test2@localhost"
        ]);
        $response->assertStatus(422);
    }
    
    public function test_update_same_login() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/auth/profile', [
            "login" => "test"
        ]);
        $response->assertStatus(204);
    }
    
    public function test_update_duplicate_login() {
        $user = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/auth/profile', [
            "username" => "test2"
        ]);
        $response->assertStatus(422);
    }
    
    public function test_update_good() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/auth/profile', [
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => 'testtest',
        ]);
        $response->assertStatus(204);
    }

    public function test_change_password() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user->markEmailAsVerified();
        
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/auth/profile', [
            'password' => 'testtest',
        ]);
        $response->assertStatus(204);

        $response = $this->postJson("/api/auth/login", [
            'email' => 'test@localhost',
            "password" => "testtest",
        ]);
        
        $response->assertStatus(200);
    }
}
